@@ -1,5 +1,5 @@
 /* Zcat - decompress and concatenate files to standard output
-   Copyright (C) 2010-2020 Antonio Diaz Diaz.
+   Copyright (C) 2010-2021 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -75,7 +75,7 @@ public:
       str[i] = '0';
       }
     if( first_digit_pos > 0 ) str[--first_digit_pos] = '1';
-    else str.insert( first_digit_pos, 1, '1' );
+    else str.insert( str.begin() + first_digit_pos, '1' );
     }
 
   int sprint( uint8_t * const buf )
