@@ -1,5 +1,5 @@
 /* Zgrep - search compressed files for a regular expression
-   Copyright (C) 2010-2020 Antonio Diaz Diaz.
+   Copyright (C) 2010-2021 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -195,7 +195,7 @@ int zgrep_file( int infd, const int format_index,
         { std::fflush( stdout ); show_error( "Write error", errno ); return 2; }
       std::fflush( stdout );
       }
-    if( size < buffer_size ) break;
+    if( size < buffer_size ) break;		// end of grep's output
     }
 
   int retval = wait_for_child( grep_pid, GREP );
