@@ -1,5 +1,5 @@
 /* Zupdate - recompress bzip2, gzip, xz files to lzip format
-   Copyright (C) 2013-2020 Antonio Diaz Diaz.
+   Copyright (C) 2013-2021 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,8 +67,8 @@ void show_help()
                "to be safe and not cause any data loss. Therefore, existing lzip\n"
                "compressed files are never overwritten nor deleted.\n"
                "\nThe names of the original files must have one of the following extensions:\n"
-               "'.bz2', '.gz', and '.xz' are recompressed to '.lz'.\n"
-               "'.tbz', '.tbz2', '.tgz', and '.txz' are recompressed to '.tlz'.\n"
+               "'.bz2', '.gz', or '.xz', which are recompressed to '.lz';\n"
+               "'.tbz', '.tbz2', '.tgz', or '.txz', which are recompressed to '.tlz'.\n"
                "\nUsage: zupdate [options] [files]\n"
                "\nExit status is 0 if all the compressed files were successfully recompressed\n"
                "(if needed), compared, and deleted (if requested). Non-zero otherwise.\n"
@@ -280,7 +280,7 @@ int zupdate_file( const std::string & name, const char * const lzip_name,
   std::string zcmp_command( invocation_name );
   unsigned i =  zcmp_command.size();
   while( i > 0 && zcmp_command[i-1] != '/' ) --i;
-  zcmp_command.resize( i ); zcmp_command.insert( 0U, 1, '\'' );
+  zcmp_command.resize( i ); zcmp_command.insert( zcmp_command.begin(), '\'' );
   zcmp_command += "zcmp' ";				// '[dir/]zcmp'
   if( no_rcfile ) zcmp_command += "-N ";
   if( verbosity < 0 ) zcmp_command += "-q ";
